import { isListType, isNonNullType } from 'graphql';
class FieldReturnTypeMismatch extends Error {
  constructor(_ref) {
    var {
      field,
      actual,
      expected
    } = _ref;
    super();
    this.name = 'FieldReturnTypeMismatch';
    var type = field.type;
    var typeDisplayName;
    if (isListType(type)) {
      typeDisplayName = "[".concat(type.ofType, "]");
    } else if (isNonNullType(type)) {
      typeDisplayName = "".concat(type.ofType, "! (non-null)");
    } else {
      typeDisplayName = type.name;
    }
    this.message = "The field \"".concat(field.name, "\" represents a graphql \"").concat(typeDisplayName, "\" type and on the document should be a ").concat(expected, ", but got ").concat(actual);
  }
}
export { FieldReturnTypeMismatch };
//# sourceMappingURL=field-return-type-mismatch.mjs.map
