'use strict';

var graphql = require('graphql');
class FieldReturnTypeMismatch extends Error {
  constructor({
    field,
    actual,
    expected
  }) {
    super();
    this.name = 'FieldReturnTypeMismatch';
    const type = field.type;
    let typeDisplayName;
    if (graphql.isListType(type)) {
      typeDisplayName = `[${type.ofType}]`;
    } else if (graphql.isNonNullType(type)) {
      typeDisplayName = `${type.ofType}! (non-null)`;
    } else {
      typeDisplayName = type.name;
    }
    this.message = `The field "${field.name}" represents a graphql "${typeDisplayName}" type and on the document should be a ${expected}, but got ${actual}`;
  }
}
exports.FieldReturnTypeMismatch = FieldReturnTypeMismatch;
//# sourceMappingURL=field-return-type-mismatch.js.map
